package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesMoveRuleService;
import cn.estsh.i3plus.ext.mes.apiservice.unit.exception.MesException;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesMoveRule;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Description : 移库规则
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/5 9:08
 * @Modify:
 **/
@Service
@Slf4j
public class MesMoveRuleImpl extends BaseMesService<MesMoveRule> implements IMesMoveRuleService {

    @Override
    protected void setPackQueryBean(MesMoveRule bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getPartProdGroupCode(), "partProdGroupCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getSrcType(), "srcType", packBean);
    }

    @Override
    protected void onInsertBean(MesMoveRule item) {
        // 数据校验
        checkNotNull(item);

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartProdGroupCode(), "partProdGroupCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(item.getSrcType(), "srcType", ddlPackBean);

        if (baseRDao.isExitByHql(ddlPackBean)) {
            MesException.throwMesBusiException("零件生产组代码【%s】来源类型【%s】数据已存在", item.getPartProdGroupCode(), item.getSrcType());
        }

    }

    @Override
    protected void onUpdateBean(MesMoveRule item) {
        // 数据校验
        checkNotNull(item);

        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getPartProdGroupCode(), "partProdGroupCode", ddlPackBean);
        DdlPreparedPack.getStringEqualPack(item.getSrcType(), "srcType", ddlPackBean);
        DdlPreparedPack.getNumNOEqualPack(item.getId(), "id", ddlPackBean);

        if (baseRDao.isExitByHql(ddlPackBean)) {
            MesException.throwMesBusiException("零件生产组代码【%s】来源类型【%s】数据已存在", item.getPartProdGroupCode(), item.getSrcType());
        }
    }

    @Override
    public List<MesMoveRule> findMesMoveRuleByPartProdGroupCode(String partProdGroupCode, String organizeCode) {
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(partProdGroupCode, "partProdGroupCode", ddlPackBean);
        List<MesMoveRule> moveRules = baseRDao.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(moveRules)) {
            MesException.throwMesBusiException("零件生产组代码【%s】移库规则数据不存在", partProdGroupCode);
        }
        return moveRules;
    }

    private void checkNotNull(MesMoveRule item) {
        ValidatorBean.checkNotNull(item.getPartProdGroupCode(), "零件生产组代码不能为空");
        ValidatorBean.checkNotNull(item.getErpSrcLocateNo(), "ERP来源库位不能为空");
        ValidatorBean.checkNotNull(item.getErpDestLocateNo(), "ERP目的库位不能为空");
        ValidatorBean.checkNotNull(item.getSrcType(), "来源类型不能为空");
        ValidatorBean.checkNotNull(item.getReportType(), "汇报类型不能为空");
    }
}
